class Customer:
    def __init__(self, name: str, customer_id: str, hosts: list = None):
        self.name = name
        self.id = customer_id
        self.hosts = hosts

    def get_details(self) -> list:
        pass

    def get_hosts(self, customer_id: str = "") -> list:
        pass

    def __str__(self):
        return self.name



import click
import site
from pathlib import Path


@click.group()
def notebooks():
    """Interact, Install or Update Jupyter notebooks """
    pass


@notebooks.command()
@click.argument('destination', type=click.Path())
def install(destination):
    """Install sample notebooks to home folder"""
    home = Path.home()
    print(home)
    print(Path(destination))
    for location in site.getsitepackages():
        path = Path(location)
        package_path = path / "seats"
        if package_path.is_dir():
            print(package_path)




from abc import ABC, abstractmethod
from requests import Response


class Connector(ABC):
    def __init__(self, target, username: str = "", password: str = ""):
        self.target = target
        self.username = username
        self.password = password

    @abstractmethod
    def login(self):
        pass

    @abstractmethod
    def logout(self):
        pass

    @abstractmethod
    def __str__(self):
        pass


class ConnectorSSH(Connector):
    @abstractmethod
    def send_command(self):
        pass


class ConnectorAPI(Connector):
    @abstractmethod
    def get(self, uri: str, options: dict) -> Response:
        pass

    @abstractmethod
    def post(self, uri: str, options: dict) -> Response:
        pass

    @abstractmethod
    def put(self, uri: str, options: dict) -> Response:
        pass

    @abstractmethod
    def delete(self, uri: str, options: dict) -> Response:
        pass



import requests
import json
from seats.connectors.connector import ConnectorAPI


class FortigateREST(ConnectorAPI):
    def __init__(self):
        self.hostname: str = ""
        self._url_prefix: str = ""
        self._debug: bool = False
        self._http_debug: bool = False
        self._https: bool = True
        self._session: requests.session = requests.session()
        self._auth_token: str = ""
        self.log_session_id: str = ""

    def __str__(self):
        return self.hostname

    def debug(self, status):
        if status == "on":
            self._debug = True
        if status == "off":
            self._debug = False

    def http_debug(self, status):
        if status == "on":
            self._http_debug = True
        if status == "off":
            self._http_debug = False

    def https(self, status):
        if status == "on":
            self._https = True
        if status == "off":
            self._https = False

    def update_csrf(self):
        # Retrieve server csrf and update session's headers
        for cookie in self._session.cookies:
            if cookie.name == "ccsrftoken":
                csrftoken = cookie.value[1:-1]  # token stored as a list
                self._session.headers.update({"X-CSRFTOKEN": csrftoken})

    def url_prefix(self, host):
        if self._https is True:
            self._url_prefix = "https://" + host
        else:
            self._url_prefix = "http://" + host

    def login_token(self, host, token):
        self._auth_token = token
        self._session.headers.update({"Authorization": "Bearer {}".format(token)})
        self._session.verify = False
        self.url_prefix(host)

    def login(self, host, username, password, timeout=None, path="/logincheck"):
        self.url_prefix(host)
        url = self._url_prefix + path
        res = self._session.post(
            url,
            data="username=" + username + "&secretkey=" + password,
            verify=False,
            timeout=timeout,
        )
        # Update session's csrftoken
        self.update_csrf()
        return res

    def logout(self, path="/logout"):
        url = self._url_prefix + path
        res = self._session.post(url)
        return res

    api_get_path = "/api/v2/"
    valid_apis = {"monitor", "cmdb", "log"}

    def get_url(self, api, path, name, action, mkey):
        # Check for valid api
        if api not in self.valid_apis:
            print("Unknown API {0}. Ignore.".format(api))
            return

        # Construct URL request, action and mkey are optional
        url_postfix = self.api_get_path + api + "/" + path + "/" + name
        if action:
            url_postfix += "/" + action
        if mkey:
            url_postfix = url_postfix + "/" + str(mkey)
        url = self._url_prefix + url_postfix
        return url

    def get(self, api, path, name, action=None, mkey=None, parameters=None):
        url = self.get_url(api, path, name, action, mkey)
        res = self._session.get(url, params=parameters)
        return res

    def post(self, api, path, name, action=None, mkey=None, parameters=None, data=None):
        url = self.get_url(api, path, name, action, mkey)
        res = self._session.post(url, params=parameters, data=json.dumps(data))
        return res

    def put(self, api, path, name, action=None, mkey=None, parameters=None, data=None):
        url = self.get_url(api, path, name, action, mkey)
        res = self._session.put(url, params=parameters, data=json.dumps(data))
        return res

    def delete(
        self, api, path, name, action=None, mkey=None, parameters=None, data=None
    ):
        url = self.get_url(api, path, name, action, mkey)
        res = self._session.delete(url, params=parameters, data=json.dumps(data))
        return res



import yaml
import json
import xmltodict
from pathlib import Path
from seats.connectors.connector import Connector


class ConnectorFile(Connector):
    def __init__(self, target: str):
        super().__init__(target)
        self.target = target
        self.file = Path(target)
        self.format = self.file.suffix

    def __str__(self):
        return self.file.name

    def login(self):
        pass

    def logout(self):
        pass

    def load_file(self):
        match self.format:
            case ".yaml":
                return yaml.safe_load(self.file)
            case ".json":
                return json.loads(self.file.read_text())
            case ".xml":
                return xmltodict.parse(self.file.read_text())
            case _:
                raise NotImplementedError(
                    f"File extension {self.format} not supported."
                )



import click
from seats.connectors.fortigate.rest import FortigateREST


@click.group()
def connector():
    """Interact with devices and services using connectors"""
    pass


@connector.command()
@click.argument('hostname')
@click.argument('username')
@click.option("--password", prompt=True, hide_input=True)
def fortigate(hostname, username, password):
    """Use fortigate REST connector"""
    fgt = FortigateREST()
    fgt.hostname = hostname
    fgt.username = username
    fgt.password = password
    print(fgt)





import click

from .cmdbs.group import cmdb
from .connectors.group import connector
from .notebooks.group import notebooks


@click.group()
def entry_point():
    """Security Engineering Automation Tool Set (SEATS)

    Collection of CLI commands using the seats package."""
    pass


# noinspection PyTypeChecker
entry_point.add_command(cmdb)
# noinspection PyTypeChecker
entry_point.add_command(connector)
# noinspection PyTypeChecker
entry_point.add_command(notebooks)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['seats',
 'seats.cli',
 'seats.cli.cmdbs',
 'seats.cli.connectors',
 'seats.cli.notebooks',
 'seats.clients',
 'seats.cmdbs',
 'seats.connectors',
 'seats.connectors.fortigate',
 'seats.models']

package_data = \
{'': ['*'], 'seats': ['notebooks/*']}

install_requires = \
['click[all,cli]>=8.1.3,<9.0.0',
 'jupyter[all,notebooks]>=1.0.0,<2.0.0',
 'pyyaml>=6.0,<7.0',
 'requests>=2.28.1,<3.0.0',
 'xmltodict>=0.13.0,<0.14.0']

entry_points = \
{'console_scripts': ['seats = seats.cli.cli:entry_point']}

setup_kwargs = {
    'name': 'seats',
    'version': '0.1.3',
    'description': 'Security Engineering Automation Tool Set',
    'long_description': '# seats\nSecurity Engineering Automation Toll Set\n',
    'author': 'Thiago Takayama',
    'author_email': 'thiago@takayama.co.uk',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)



import click
from seats.cmdbs.file import FromFile


@click.group()
def cmdb():
    """Interact with customer databases"""
    pass


@cmdb.command()
@click.argument('config', type=click.File())
def file(config):
    """Loads cmdb information from file"""
    srv = FromFile(config_path=config)
    customers = srv.get_customers()
    for customer in customers:
        print(customer)
        print(customer.hosts)



from seats.cmdbs.cmdb import CMDB
from seats.connectors.file import ConnectorFile
from seats.models.customer import Customer


class FromFile(CMDB):
    """
    Service defined in file.

    Attributes
    ----------

    config : str


    Methods
    -------

    """

    def __init__(
        self, config: dict = None, config_path: str = "config.json"
    ):
        """
        Parameters
        ----------
        config : None
            Is not used for this connector and is set to None by default
        config_path: str
            The default value is 'config.json'
            Must point to a valid cmdb file.
            The file must be in JSON, YAML or XML formats.
            The file should have the extension .json, .yaml or .xml.
            The file should contain a list of customers.
            For an example of a valid file check notebooks/cmdb.json
        """
        super().__init__(config, config_path)
        self.config = config
        self.config_path = config_path
        self.connector = ConnectorFile(self.config_path)
        self.file = self.connector.load_file()
        self.customers = self.get_customers()

    def __str__(self):
        return f"{self.config_path}: {self.connector}"

    def get_customers(self) -> list:
        """
        Iterates over provided file to retrieve customers information.
        Loads name, id and hosts for customer and instantiates a Customer.

        returns: List of Customers
        """
        if type(self.file) != list:
            raise Exception("File does not seem to contain a valid list")
        customers = []
        for customer in self.file:
            customers.append(
                Customer(
                    name=customer.get("name"),
                    customer_id=customer.get("id"),
                    hosts=customer.get("hosts"),
                )
            )
        return customers

    def get_customer_by_name(self, customer_name: str) -> dict:
        return next(x for x in self.customers if x.get("name") == customer_name)

    def get_customer_by_id(self, customer_id: str) -> dict:
        return next(x for x in self.customers if x.get("id") == customer_id)

    def get_hosts(self, customer_id: str) -> list:
        customer = next(x for x in self.customers if x.id == customer_id)
        return customer.hosts

    def get_host_config(self, customer_id: str, host_id: str) -> str:
        hosts = self.get_hosts(customer_id)
        host = next(x for x in hosts if x.get("id") == host_id)
        return host.get("config")



from abc import ABC, abstractmethod


class CMDB(ABC):
    def __init__(
        self, config: dict = None, config_path: str = "config.json"
    ):
        self.config = config
        self.config_path = config_path

    @abstractmethod
    def get_customers(self) -> list:
        pass

    @abstractmethod
    def get_customer_by_name(self, customer_name: str):
        pass

    @abstractmethod
    def get_customer_by_id(self, customer_id: str):
        pass

    @abstractmethod
    def get_hosts(self, customer_id: str) -> list:
        pass

    @abstractmethod
    def get_host_config(self, customer_id: str, host_id: str) -> str:
        pass
